/* 
 * File:   main.h
 * Author: zacharymurtishi
 *
 * Created on December 9, 2024, 4:55 PM
 */

#ifndef MAIN_H
#define	MAIN_H

#ifdef	__cplusplus
extern "C" {
#endif

#define SW1 ((buttonState>>6)&0b1) 
#define SW2 ((buttonState>>7)&0b1) 
#define SW3 ((buttonState>>3)&0b1) 
#define SW4 ((buttonState>>4)&0b1) 
#define SW5 ((buttonState>>1)&0b1) 
#define SW6 ((buttonState>>2)&0b1) 
#define SW7 ((buttonState)&0b1) 
#define SW8 ((buttonState>>5)&0b1)

#define AM_LED (1<<4)
#define PM_LED (1<<5)
    
#define MAX_HOURS_AMPM 12
    
#define NULL_CHAR 15

#define HOURS24 1
#define AMPM 0

#define MAX_TICKS 1000

#define MAX_HOURS 24 
#define MAX_MINUTES 60
#define MAX_SECONDS 60
#define MAX_DAYS 7

#define SECONDS_OFFSET_CLOCK 0
#define MINUTES_OFFSET_CLOCK 1
#define HOURS_OFFSET_CLOCK 2
#define DAYS_OFFSET_CLOCK 3
    
#define MINUTES_OFFSET_ALARM 0
#define HOURS_OFFSET_ALARM 1
    
#define DIGIT_ONE 0
#define DIGIT_TWO 1
#define DIGIT_THREE 2
#define DIGIT_FOUR 3
    
#define ALARM_SET_LED (1<<3)
#define BUZZER (1<<1)
    
#define MINUTES_TO_SNOOZE 5
#define FIVEHUNDREDHZ 65473
#define ONETHOUSANDHZ 65488
    
#define DIGITREFRESHINTERVAL 4
#define BUTTONSCANINTERVAL 16
    
#define NORMAL_MODE 0
#define DIMMED_MODE 1
#define LAMPTEST_MODE 2
#define SECONDS_MODE 3
    
typedef volatile uint8_t vuint8_t;
typedef volatile uint16_t vuint16_t;
    
void timeRefresh(vuint8_t alarmMode, vuint8_t *digitArr, vuint8_t *clockTime, vuint8_t *alarmTime, vuint8_t hourMode){  
    uint8_t adjustedHours;
    uint8_t adjustedMinutes;
    if(alarmMode){
        adjustedHours = *(alarmTime+HOURS_OFFSET_ALARM);
        adjustedMinutes = *alarmTime;
    }
    else{
        adjustedHours = *(clockTime+HOURS_OFFSET_CLOCK);
        adjustedMinutes = *(clockTime+MINUTES_OFFSET_CLOCK);        
    }
    if((adjustedHours == 0) && (hourMode==AMPM)){
        adjustedHours = adjustedHours + MAX_HOURS_AMPM;              
    }
    else if((adjustedHours >= MAX_HOURS_AMPM) && (hourMode==AMPM)){
        if(adjustedHours != MAX_HOURS_AMPM){
            adjustedHours = adjustedHours - MAX_HOURS_AMPM;
        }
        else{
            adjustedHours = adjustedHours;
        }
    }
    else if((adjustedHours < MAX_HOURS_AMPM) && (hourMode==AMPM)){
        adjustedHours = adjustedHours;        
    }
    else{
        adjustedHours = adjustedHours;
    }
    *(digitArr+DIGIT_ONE) = adjustedMinutes % 10;
    *(digitArr+DIGIT_TWO) = (adjustedMinutes/10) % 10;
    *(digitArr+DIGIT_THREE) = adjustedHours % 10;
    *(digitArr+DIGIT_FOUR) = (adjustedHours/10) % 10;   
}
    
void digitRefresh(vuint8_t *displayMode, vuint8_t *digitN, vuint8_t *digitArr, vuint8_t *flashingDigits, vuint8_t numSeconds){
    *digitN = ((*digitN) + 1)&0b11;          
    if((*displayMode == DIMMED_MODE) || *flashingDigits){
            PORTD |= (uint8_t)((*digitN)<<6);          
            PORTD |= (uint8_t)((NULL_CHAR)<<2);
    }
    else if(*displayMode == LAMPTEST_MODE){
        PORTD = (uint8_t)((*digitN)<<6);          
        if(*digitN == 0){
            PORTD |= (uint8_t)(8<<2);            
        }
        else if(*digitN == 1){
            PORTD |= (uint8_t)(8<<2);            
        }
        else if(*digitN == 2){
            PORTD |= (uint8_t)(8<<2);            
        }
        else if(*digitN == 3){
            PORTD |= (uint8_t)(8<<2);            
        }        
    }
    else if(*displayMode == SECONDS_MODE){
        PORTD = (uint8_t)((*digitN)<<6);          
        if(*digitN == 0){
            PORTD |= (uint8_t)(NULL_CHAR<<2);            
        }
        else if(*digitN == 1){
            PORTD |= (uint8_t)(((numSeconds/10)%10)<<2);            
        }
        else if(*digitN == 2){
            PORTD |= (uint8_t)(NULL_CHAR<<2);            
        }
        else if(*digitN == 3){
            PORTD |= (uint8_t)((numSeconds%10)<<2);            
        }        
    }    
    else{   
        PORTD = (uint8_t)((*digitN)<<6);          
        if(*digitN == 0){
            PORTD |= (uint8_t)((*(digitArr+DIGIT_FOUR)%10)<<2);            
        }
        else if(*digitN == 1){
            PORTD |= (uint8_t)((*(digitArr+DIGIT_TWO)%10)<<2);            
        }
        else if(*digitN == 2){
            PORTD |= (uint8_t)((*(digitArr+DIGIT_THREE)%10)<<2);            
        }
        else if(*digitN == 3){
            PORTD |= (uint8_t)((*(digitArr+DIGIT_ONE)%10)<<2);            
        } 
    }
}

uint8_t readButtons(void){
    uint8_t buttonsIn = 0x00;
    PORTC &= 0b11110111;
    PORTC &= 0b11111011;
    asm("NOP");   
    asm("NOP");   
    asm("NOP");   
    PORTC |= 0b00000100;
    asm("NOP");
    asm("NOP");   
    asm("NOP");     
    PORTC &= 0b11111011;    
    for(uint8_t i=0; i<8; i++){
        buttonsIn = buttonsIn<<1;        
        buttonsIn |= ((PINC>>1)&1);        
        PORTC &= 0b11110111;
        asm("NOP");  
        asm("NOP");   
        asm("NOP");         
        PORTC |= 0b00001000;
        asm("NOP");
        asm("NOP");   
        asm("NOP");         
        PORTC &= 0b11110111;
    } 
    buttonsIn = ~buttonsIn;
    return buttonsIn;
}  

void stopAlarm(vuint8_t *alarmTriggered, vuint8_t *displayMode){
    *alarmTriggered = 0;
    *displayMode = NORMAL_MODE;
    TIMSK1 |= 0b000;    
}

void snooze(vuint8_t *alarmTriggered, vuint8_t *snoozeSet, vuint8_t *snoozeMinutes, vuint8_t *snoozeHours, vuint8_t *clockTime){
    *alarmTriggered = 0;
    TIMSK1 |= 0b000;
    *snoozeSet = 1;
    *snoozeMinutes = *(clockTime+MINUTES_OFFSET_CLOCK) + MINUTES_TO_SNOOZE;
    *snoozeHours = *(clockTime+HOURS_OFFSET_CLOCK);
    if(*snoozeMinutes >= MAX_MINUTES){
        *snoozeHours = *snoozeHours + 1;
        if(*snoozeHours >= MAX_HOURS){
            *snoozeHours = *snoozeHours - MAX_HOURS;
        }        
        *snoozeMinutes = *snoozeMinutes - MAX_MINUTES;
    }    
}
    
void parseButtons(vuint8_t *lastState, vuint8_t *alarmMode, vuint8_t *alarmTriggered, vuint8_t *alarmSet, vuint8_t *displayMode, vuint8_t *alarmTime, vuint8_t *clockTime, vuint8_t *snoozeSet, vuint8_t *hourMode, vuint8_t *snoozeMinutes, vuint8_t *snoozeHours){
        uint8_t buttonState = readButtons();
        if(buttonState != (*lastState)){
            if(SW1){
                if(*alarmTriggered){
                    snooze(alarmTriggered, snoozeSet, snoozeMinutes, snoozeHours, clockTime);
                }
                else if(*displayMode != NORMAL_MODE){
                    *displayMode = NORMAL_MODE;
                }                
                else{
                    if(*alarmMode){
                        *(alarmTime+HOURS_OFFSET_ALARM) = *(alarmTime+HOURS_OFFSET_ALARM)+1;
                        if(*(alarmTime+HOURS_OFFSET_ALARM) == MAX_HOURS){
                            *(alarmTime+HOURS_OFFSET_ALARM) = 0;
                        }
                    }                 
                    else{
                        *(clockTime+HOURS_OFFSET_CLOCK) = *(clockTime+HOURS_OFFSET_CLOCK) + 1;
                        if(*(clockTime+HOURS_OFFSET_CLOCK) == MAX_HOURS){
                            *(clockTime+HOURS_OFFSET_CLOCK) = 0;
                        }
                    }
                }
            }            
            else if(SW2){
                if(*alarmTriggered){          
                    snooze(alarmTriggered, snoozeSet, snoozeMinutes, snoozeHours, clockTime);
                }
                else if(*displayMode != NORMAL_MODE){
                    *displayMode = NORMAL_MODE;
                }                 
                else{
                    if(*(alarmMode)){
                        *(alarmTime) = *(alarmTime)+1; 
                        if(*(alarmTime) == MAX_MINUTES){
                            *(alarmTime) = 0;
                        }
                    }                 
                    else{
                        *(clockTime+MINUTES_OFFSET_CLOCK) = *(clockTime+MINUTES_OFFSET_CLOCK) + 1;
                        if(*(clockTime+MINUTES_OFFSET_CLOCK) == MAX_MINUTES){
                            *(clockTime+MINUTES_OFFSET_CLOCK) = 0;
                        }
                        *(clockTime)=0;
                    }       
                }
            }
            else if(SW3){
                if(*alarmTriggered){
                    snooze(alarmTriggered, snoozeSet, snoozeMinutes, snoozeHours, clockTime);
                }
                else if(*displayMode != NORMAL_MODE){
                    *displayMode = NORMAL_MODE;
                }               
                else{
                    *hourMode ^= 1;                  
                }
            }
            else if(SW4){
                if(*alarmTriggered){
                    snooze(alarmTriggered, snoozeSet, snoozeMinutes, snoozeHours, clockTime);
                }
                else if(*displayMode != NORMAL_MODE){
                    *displayMode = NORMAL_MODE;
                }                
                else{
                     *(clockTime+DAYS_OFFSET_CLOCK) = *(clockTime+DAYS_OFFSET_CLOCK) + 1;
                     if(*(clockTime+DAYS_OFFSET_CLOCK) == MAX_DAYS){
                         *(clockTime+DAYS_OFFSET_CLOCK) = 0;
                     }
                }
            }
            else if(SW5){
                if(*alarmTriggered){
                    snooze(alarmTriggered, snoozeSet, snoozeMinutes, snoozeHours, clockTime);
                }
                else if(*displayMode != NORMAL_MODE){
                    *displayMode = NORMAL_MODE;
                }               
                else{
                    *alarmSet = 0;
                    *alarmMode ^= 1;
                    if(*alarmMode == 0){
                        DDRD = 0xFF;
                    }
                }
            }
            else if(SW6){
                if(*alarmTriggered){
                    snooze(alarmTriggered, snoozeSet, snoozeMinutes, snoozeHours, clockTime);
                }
                else if(*displayMode != NORMAL_MODE){
                    *displayMode = NORMAL_MODE;
                }              
                else{
                    if(!(*alarmMode)){
                        *alarmSet ^= 1;
                        if(!(*alarmSet))
                        {
                            *snoozeSet = 0;
                        }
                    }
                }
            }
            else if(SW7){
                if(*alarmTriggered){
                    snooze(alarmTriggered, snoozeSet, snoozeMinutes, snoozeHours, clockTime);
                }
                else if(*displayMode != NORMAL_MODE){
                    *displayMode = NORMAL_MODE;
                }
                else{
                    if(!(*alarmMode)){
                        *displayMode = DIMMED_MODE;
                    }
                }             
            }
            else if(SW8){
                if(*alarmTriggered){
                    stopAlarm(alarmTriggered, displayMode);
                }                
                else if(*displayMode == DIMMED_MODE){
                    *displayMode = LAMPTEST_MODE;
                }           
                else if((*displayMode == LAMPTEST_MODE) || (*displayMode == SECONDS_MODE)){
                    *displayMode = NORMAL_MODE;
                }
                else{
                    *displayMode = SECONDS_MODE;
                }
            }
        }
        *lastState = buttonState;    
}    

void updateLEDs(vuint8_t displayMode, vuint8_t hourMode, vuint8_t alarmSet, vuint8_t alarmMode, vuint8_t *alarmTime, vuint8_t *clockTime, vuint8_t systemTicks){
    if(displayMode == DIMMED_MODE)
    {
        PORTE &= 0b0111;
        PORTE |= 0b0111;
        PORTC &= 0b11001111;
    }
    else if(displayMode == LAMPTEST_MODE){
        PORTC |= 0b00110000;
        PORTE = 0b1000 | (systemTicks%MAX_DAYS);
    }
    else{
        if(alarmSet){
            PORTE |= ALARM_SET_LED;
        }
        else{
            PORTE &= ~ALARM_SET_LED;
        }
        PORTE &= 0b1000;
        PORTE |= *(clockTime+DAYS_OFFSET_CLOCK);
    
        if(hourMode == HOURS24){
            PORTC &= ~PM_LED;
            PORTC &= ~AM_LED;
        }
        else{
            if(alarmMode){
                if(*(alarmTime+HOURS_OFFSET_ALARM) >= MAX_HOURS_AMPM){
                    PORTC |= PM_LED;
                    PORTC &= ~AM_LED;
                }
                else{
                    PORTC |= AM_LED;
                    PORTC &= ~PM_LED;
                }                
            }
            else{
                if(*(clockTime+HOURS_OFFSET_CLOCK) >= MAX_HOURS_AMPM){
                    PORTC |= PM_LED;
                    PORTC &= ~AM_LED;
                }
                else{
                    PORTC |= AM_LED;
                    PORTC &= ~PM_LED;
                }
            }
        }        
    }
}


#ifdef	__cplusplus
}
#endif

#endif	/* MAIN_H */

