/*
 * File:   newmain1.c
 * Author: zacharymurtishi
 *
 * Created on December 4, 2024, 5:14 PM
 */


// ATmega328PB Configuration Bit Settings

// 'C' source line config statements

#include <avr/io.h>
#include <avr/interrupt.h>
#include "alarm.h"
#include "usart.h"

#define FOSC 8000000 // Clock Speed

FUSES = {
	.low = 0xE2, // LOW {SUT_CKSEL=INTRCOSC_8MHZ_6CK_14CK_65MS, CKOUT=CLEAR, CKDIV8=CLEAR}
	.high = 0xD9, // HIGH {BOOTRST=CLEAR, BOOTSZ=2048W_3800, EESAVE=CLEAR, WDTON=CLEAR, SPIEN=SET, DWEN=CLEAR, RSTDISBL=CLEAR}
	.extended = 0xF7, // EXTENDED {BODLEVEL=DISABLED, CFD=CFD_DISABLED}
};

LOCKBITS = 0xFF; // {LB=NO_LOCK, BLB0=NO_LOCK, BLB1=NO_LOCK}

vuint16_t ticks = 0;

vuint8_t digitNumber[4] = {0,0,0,0};

vuint8_t digit = 0;

vuint8_t lastState = 0;

vuint8_t globalTime[4] = {0,0,0,0};

vuint8_t globalAlarmTime[2] = {0,0};

vuint8_t globalHourMode = 0;

vuint8_t globalAlarmMode = 0;
vuint8_t globalAlarmSet = 0;
vuint8_t globalAlarmTriggered = 0;

vuint8_t globalSnoozeSet = 0;
vuint8_t globalSnoozeHours = 0;
vuint8_t globalSnoozeMinutes = 0;

vuint8_t globalFlashingDigits = 0;

vuint8_t serialRingBuffer[MAX_ITEMS];
vuint8_t head = 0;
vuint8_t items = 0;

vuint8_t globalDisplayMode = NORMAL_MODE;

vuint8_t serialTimeout = 0;

static uint8_t testWord[6] = {'T','E','S','T','\n','\r'};  
static uint8_t errorWord[7] = {'E','R','R','O','R','\n','\r'};  
static uint8_t doneWord[6] = {'D','O','N','E','\n','\r'};  
#define SUCCESS USART_Transmit(doneWord, sizeof(doneWord));
#define ERROR USART_Transmit(errorWord, sizeof(errorWord));

ISR(USART0_RX_vect, ISR_BLOCK){
    uint8_t dataIn = 0;
    while (!(UCSR0A & (1<<RXC0)));
    dataIn = UDR0;
    serialTimeout = 0;
    insert(dataIn, &items, &head, serialRingBuffer);   
}

ISR(TIMER0_OVF_vect, ISR_BLOCK){
    TCNT0 = 248;
    ticks++;
    serialTimeout++;
    if(ticks==MAX_TICKS){
        ticks=0;
    }
    if((ticks%DIGITREFRESHINTERVAL) == 0){
        digitRefresh(&globalDisplayMode, &digit, digitNumber, &globalFlashingDigits, globalTime[0]);
        updateLEDs(globalDisplayMode, globalHourMode, globalAlarmSet, globalAlarmMode, globalAlarmTime, globalTime, ticks);
        timeRefresh(globalAlarmMode, digitNumber, globalTime, globalAlarmTime, globalHourMode);    
    }
    if((ticks%BUTTONSCANINTERVAL) == 0){ 
        parseButtons(&lastState, &globalAlarmMode, &globalAlarmTriggered, &globalAlarmSet, &globalDisplayMode, globalAlarmTime, globalTime, &globalSnoozeSet,&globalHourMode,&globalSnoozeMinutes,&globalSnoozeHours);
    }        
    if((ticks == 250)){
        if(globalAlarmMode || globalAlarmTriggered){
            globalFlashingDigits = 1;
        }
    }
    if((ticks == 500) || (ticks==0)){
        if(globalAlarmTriggered){
            TIMSK1 ^= 0b001;
        }
        else{
            TIMSK1 = 0b000;
            PORTB &= ~BUZZER;
        }        
    }
    if((ticks == 500)){
        globalFlashingDigits = 0;
    }    
}

ISR(TIMER1_OVF_vect, ISR_BLOCK){
    PORTB ^= BUZZER;
    TCNT1 = FIVEHUNDREDHZ;
}

ISR(TIMER2_OVF_vect, ISR_BLOCK){
    globalTime[0]++;
    if(globalTime[0] == MAX_SECONDS){
        globalTime[0] = 0;
        globalTime[1]++;   
        if(globalTime[1] == MAX_MINUTES){
            globalTime[1]=0;
            globalTime[2]++;
        }    
        if(globalTime[2] == MAX_HOURS){
            globalTime[2] = 0;
            globalTime[3]++;
            if(globalTime[3] == MAX_DAYS){
                globalTime[3] = 0;
            }
        }        
        if(globalAlarmSet){
            if(globalAlarmTime[0] == globalTime[1]){
                if(globalAlarmTime[1] == globalTime[2]){   
                    TIMSK1 |= 0b001;
                    globalAlarmMode = 0;
                    globalAlarmTriggered = 1;
                    globalDisplayMode = NORMAL_MODE;
                }
            }
            else if(globalSnoozeSet){
                if(globalSnoozeMinutes == globalTime[1]){
                    if(globalSnoozeHours == globalTime[2]){
                        TIMSK1 |= 0b001;
                        globalAlarmMode = 0;
                        globalAlarmTriggered = 1;
                        globalDisplayMode = NORMAL_MODE;
                    }
                }
            }
        }
    }
}

void portInit(void){
    DDRB = 0b00000010;
    DDRD = 0xFF;
    DDRC = 0b00111100;
    DDRE = 0xFF;
    PORTC=0;
    PORTE=0;
    PORTD=0;
    PORTB=0;
}

void timerInit(void){
    TIMSK0 |= 0b001;
    TCCR0B |= 0b101;
    TCNT0 = 248;
    SREG |= (1<<7);
    
    TCCR1A = 0b00000000;
    TCCR1B = 0b00000100;
    TIMSK1 |= 0b000;
    TCNT1 = FIVEHUNDREDHZ;
    
    TCCR2B |= 0b101;
    TCNT2 = 0;
    TIMSK2 |= 0b001;
    ASSR |= (1<<5);
}

int main(void) {
    USART_Init(BAUDRATE);
    timeRefresh(globalAlarmMode, digitNumber, globalTime, globalAlarmTime, globalHourMode);   
    portInit();
    timerInit();
    uint8_t data = 0; 
    uint8_t hoursMinutes[3] = {0,0,0};
    while(1){  
        if(items){
            data = pop(&items, &head, serialRingBuffer);
            if(data == QUERY){  
                USART_Transmit(globalTime, sizeof(globalTime));             
            }
            else if(data == SET){             
                vuint8_t itemsReceived = 0;
                serialTimeout = 0;
                while((serialTimeout < TIMEOUT_10MS) && (itemsReceived < 3)){
                    if(items){
                        hoursMinutes[itemsReceived] = pop(&items, &head, serialRingBuffer);
                        itemsReceived++;
                    }
                }
                if(itemsReceived == 3){
                    if(((hoursMinutes[0] < MAX_HOURS) && (hoursMinutes[1] < MAX_MINUTES)) && (hoursMinutes[2] < MAX_DAYS)){
                        globalTime[0] = 0;
                        globalTime[2] = hoursMinutes[0];
                        globalTime[1] = hoursMinutes[1];
                        globalTime[3] = hoursMinutes[2];
                        SUCCESS                     
                    }
                    else{
                        ERROR                    
                    }                    
                }
                else{
                    ERROR                    
                }
            }
            else if(data == UPDATE_ALARM){
                volatile uint8_t itemsReceived = 0;
                serialTimeout = 0;
                while((serialTimeout < TIMEOUT_10MS) && (itemsReceived < 2)){
                    if(items){
                        hoursMinutes[itemsReceived] = pop(&items, &head, serialRingBuffer);
                        itemsReceived++;
                    }
                }
                if(itemsReceived == 2){
                    if((hoursMinutes[0] < MAX_HOURS) && (hoursMinutes[1] < MAX_MINUTES)){
                        globalAlarmTime[1] = hoursMinutes[0];
                        globalAlarmTime[0] = hoursMinutes[1];
                        SUCCESS  
                    }
                    else{
                        ERROR                    
                    }                        
                }
                else{
                    ERROR                     
                }                 
            }                
            else if(data == ARM_ALARM){
                if((!globalAlarmMode)&&(globalAlarmSet==0)){
                    globalAlarmSet = 1;
                    SUCCESS                    
                }
                else{
                    ERROR
                }
            }
            else if(data == DISARM_ALARM){
                if(globalAlarmSet==1){
                    globalAlarmSet = 0;
                    SUCCESS     
                }
                else{
                    ERROR
                }
            }            
            else if(data == SNOOZE_ALARM){
                if(globalAlarmTriggered){
                    snooze(&globalAlarmTriggered, &globalSnoozeSet, &globalSnoozeMinutes, &globalSnoozeHours, globalTime);
                    SUCCESS                     
                }
                else{
                    ERROR                     
                }
            }
            else if(data == STOP_ALARM){
                if(globalAlarmTriggered){
                    stopAlarm(&globalAlarmTriggered, &globalDisplayMode);
                    SUCCESS                        
                }
                else{
                    ERROR                 
                }
            }               
            else if(data == TEST){
                USART_Transmit(testWord, sizeof(testWord));
            }
            else if(data == DIM_LIGHTS){
                if(globalDisplayMode != DIMMED_MODE){
                    globalDisplayMode = DIMMED_MODE;
                    SUCCESS
                }
                else{
                    ERROR
                }
            } 
            else if(data == NORMAL_LIGHTS){
                if(globalDisplayMode != NORMAL_MODE){
                    globalDisplayMode = NORMAL_MODE;
                    SUCCESS
                }
                else{
                    ERROR
                }    
            }
            else if(data == LAMPTEST){
                if(globalDisplayMode != LAMPTEST_MODE){
                    globalDisplayMode = LAMPTEST_MODE;
                    SUCCESS
                }
                else{
                    ERROR
                }    
            }          
            else if(data == HOURMODE){
                globalHourMode ^= 1;
                SUCCESS
            }     
            else if(data == SECONDS){
                if(globalDisplayMode != SECONDS_MODE){
                    globalDisplayMode = SECONDS_MODE;
                    SUCCESS
                }
                else{
                    ERROR
                } 
            }              
            else{
                ERROR
            }
        }
    }    
}
