/* 
 * File:   usart.h
 * Author: zacharymurtishi
 *
 * Created on December 11, 2024, 5:27 PM
 */

#ifndef USART_H
#define	USART_H

#ifdef	__cplusplus
extern "C" {
#endif

#define BAUDRATE 12
#define MAX_ITEMS 16
#define TIMEOUT_10MS 10    
    
#define QUERY 'A'
#define SET 'B'
#define TEST 'C'
#define ARM_ALARM 'D'
#define DISARM_ALARM 'E'
#define UPDATE_ALARM 'F'
#define SNOOZE_ALARM 'G'
#define STOP_ALARM 'H'
#define DIM_LIGHTS 'I'
#define LAMPTEST 'J'
#define NORMAL_LIGHTS 'K'
#define HOURMODE 'L' 
#define SECONDS 'M'
     
typedef volatile uint8_t vuint8_t;
typedef volatile uint16_t vuint16_t;
    
void insert(uint8_t value, vuint8_t *numItems, vuint8_t *bufferHead, vuint8_t *buffer){
    *(buffer+(*bufferHead % MAX_ITEMS)) = value;
    *bufferHead = (*(bufferHead)+1)%MAX_ITEMS;
    if(*numItems < MAX_ITEMS){
        *numItems = *numItems + 1;
    }
}

uint8_t pop(vuint8_t *numItems, vuint8_t *bufferHead, vuint8_t *buffer){
    *bufferHead = *bufferHead - 1;
    uint8_t popped_val = *(buffer+(*bufferHead % MAX_ITEMS));
    if(*numItems){
        *(numItems) = *(numItems) - 1;
    }
    return popped_val;
}

void USART_Init(uint16_t baudrate)
{
    /*Set baud rate */
    UBRR0H = (uint8_t)(baudrate>>8);
    UBRR0L = (uint8_t)(baudrate&0xFF);
    UCSR0B = 0b10011000;
    UCSR0C = 0b00000110;
}

void USART_Transmit(vuint8_t *data, uint8_t length)
{
    for(uint8_t i=0; i<length; i++){
        while (!(UCSR0A&(1<<UDRE0)));  
        UDR0 = *(data+i);
    }
}


#ifdef	__cplusplus
}
#endif

#endif	/* USART_H */

